@@ -60,6 +60,75 @@ START_TEST(test_next_event)
 }
 END_TEST
 
+START_TEST(test_next_event_invalid_fd)
+{
+	struct uinput_device* uidev;
+	struct libevdev *dev;
+	int rc;
+	struct input_event ev;
+
+	libevdev_set_log_function(test_logfunc_ignore_error, NULL);
+
+	dev = libevdev_new();
+	rc = libevdev_next_event(dev, LIBEVDEV_READ_FLAG_NORMAL, &ev);
+	ck_assert_int_eq(rc, -EBADF);
+	libevdev_free(dev);
+
+	test_create_device(&uidev, &dev,
+			   EV_REL, REL_X,
+			   EV_REL, REL_Y,
+			   EV_KEY, BTN_LEFT,
+			   -1);
+
+	/* invalid (missing) flag */
+	rc = libevdev_next_event(dev, 0x10, &ev);
+	ck_assert_int_eq(rc, -EINVAL);
+
+	/* set an invalid fd */
+	rc = libevdev_change_fd(dev, -3);
+	ck_assert_int_eq(rc, 0);
+	rc = libevdev_next_event(dev, LIBEVDEV_READ_FLAG_NORMAL, &ev);
+	ck_assert_int_eq(rc, -EBADF);
+
+	libevdev_set_log_function(test_logfunc_abort_on_error, NULL);
+
+	libevdev_free(dev);
+	uinput_device_free(uidev);
+}
+END_TEST
+
+START_TEST(test_next_event_blocking)
+{
+	struct uinput_device* uidev;
+	struct libevdev *dev;
+	int fd, flags;
+	int rc;
+	struct input_event ev;
+
+	test_create_device(&uidev, &dev,
+			   EV_REL, REL_X,
+			   EV_REL, REL_Y,
+			   EV_KEY, BTN_LEFT,
+			   -1);
+
+	fd = libevdev_get_fd(dev);
+	flags = fcntl(fd, F_GETFL) & ~O_NONBLOCK;
+	rc = fcntl(fd, F_SETFL, flags);
+	ck_assert_int_eq(rc, 0);
+
+	uinput_device_event(uidev, EV_KEY, BTN_LEFT, 1);
+	uinput_device_event(uidev, EV_SYN, SYN_REPORT, 0);
+	rc = libevdev_next_event(dev, LIBEVDEV_READ_FLAG_BLOCKING, &ev);
+	ck_assert_int_eq(rc, LIBEVDEV_READ_STATUS_SUCCESS);
+	ck_assert_int_eq(ev.type, EV_KEY);
+	ck_assert_int_eq(ev.code, BTN_LEFT);
+	ck_assert_int_eq(ev.value, 1);
+
+	libevdev_free(dev);
+	uinput_device_free(uidev);
+}
+END_TEST
+
 START_TEST(test_syn_dropped_event)
 {
 	struct uinput_device* uidev;
@@ -342,6 +411,23 @@ START_TEST(test_has_event_pending)
 }
 END_TEST
 
+START_TEST(test_has_event_pending_invalid_fd)
+{
+	struct libevdev *dev;
+	int rc;
+
+	libevdev_set_log_function(test_logfunc_ignore_error, NULL);
+
+	dev = libevdev_new();
+	rc = libevdev_has_event_pending(dev);
+	ck_assert_int_eq(rc, -EBADF);
+
+	libevdev_set_log_function(test_logfunc_abort_on_error, NULL);
+
+	libevdev_free(dev);
+}
+END_TEST
+
 START_TEST(test_syn_delta_button)
 {
 	struct uinput_device* uidev;
@@ -479,7 +565,6 @@ START_TEST(test_syn_delta_mt)
 	abs[3].value = ABS_MT_POSITION_Y;
 	abs[3].maximum = 1000;
 
-
 	abs[4].value = ABS_MT_SLOT;
 	abs[4].maximum = 1;
 	abs[5].value = ABS_MT_TRACKING_ID;
@@ -594,7 +679,6 @@ START_TEST(test_syn_delta_mt_reset_slot)
 	abs[3].value = ABS_MT_POSITION_Y;
 	abs[3].maximum = 1000;
 
-
 	abs[4].value = ABS_MT_SLOT;
 	abs[4].maximum = 1;
 	abs[5].value = ABS_MT_TRACKING_ID;
@@ -1055,7 +1139,6 @@ START_TEST(test_syn_delta_late_sync)
 		}
 	}
 
-
 	/* Now we basically re-do the exact same test, just with the
 	   tracking ID order inverted */
 
@@ -1139,7 +1222,6 @@ START_TEST(test_syn_delta_late_sync)
 		}
 	}
 
-
 	uinput_device_free(uidev);
 	libevdev_free(dev);
 }
@@ -1446,7 +1528,6 @@ START_TEST(test_event_values_invalid)
 	ck_assert_int_eq(libevdev_fetch_event_value(dev, EV_REL, REL_Z, &value), 0);
 	ck_assert_int_eq(value, 0xab);
 
-
 	uinput_device_free(uidev);
 	libevdev_free(dev);
 
@@ -2066,11 +2147,14 @@ libevdev_events(void)
 
 	TCase *tc = tcase_create("event polling");
 	tcase_add_test(tc, test_next_event);
+	tcase_add_test(tc, test_next_event_invalid_fd);
+	tcase_add_test(tc, test_next_event_blocking);
 	tcase_add_test(tc, test_syn_dropped_event);
 	tcase_add_test(tc, test_double_syn_dropped_event);
 	tcase_add_test(tc, test_event_type_filtered);
 	tcase_add_test(tc, test_event_code_filtered);
 	tcase_add_test(tc, test_has_event_pending);
+	tcase_add_test(tc, test_has_event_pending_invalid_fd);
 	suite_add_tcase(s, tc);
 
 	tc = tcase_create("SYN_DROPPED deltas");
@@ -2112,4 +2196,3 @@ libevdev_events(void)
 
 	return s;
 }
-
